@@ -84,7 +84,7 @@ class AdsorbateOperator(OffspringCreator):
             should be found.
         
         tilt_angle : float, default 0.
-            Tilt the adsorbate with an angle (in degress) relative to the 
+            Tilt the adsorbate with an angle (in degrees) relative to the 
             surface normal.
 
         """
@@ -164,8 +164,8 @@ class AdsorbateOperator(OffspringCreator):
         """
 
         i = 0
-        occupied = True
-        while occupied:
+        occupied = False
+        while not occupied:
             site = hetero_site_list[i]
             if not site['occupied']:
                 i += 1
@@ -179,13 +179,12 @@ class AdsorbateOperator(OffspringCreator):
 
             # Remove adsorbate from the correct position. Remove the fragment
             # with priority if it is one of the given adsorbate species
-            rm_frag = site['fragment'] in self.adsorbate_species 
+            rm_frag = (site['fragment'] in self.adsorbate_species) 
             remove_adsorbate_from_site(atoms, site, remove_fragment=rm_frag)
 
             if return_site_index:
                 return i
-
-            occupied = False
+            occupied = True
 
         return atoms
 
@@ -295,7 +294,7 @@ class AddAdsorbate(AdsorbateOperator):
         if max_coverage is not specified.
 
     tilt_angle : float, default 0.
-        Tilt the adsorbate with an angle (in degress) relative to the 
+        Tilt the adsorbate with an angle (in degrees) relative to the 
         surface normal.
 
     subtract_heights : bool, default False
@@ -614,7 +613,7 @@ class MoveAdsorbate(AdsorbateOperator):
         Please only use this for nanoparticles.
 
     tilt_angle : float, default 0.
-        Tilt the adsorbate with an angle (in degress) relative to the 
+        Tilt the adsorbate with an angle (in degrees) relative to the 
         surface normal.
 
     subtract_heights : bool, default False
@@ -721,7 +720,7 @@ class MoveAdsorbate(AdsorbateOperator):
 
             removed = self.remove_adsorbate(indi, ads_sites,
                                             return_site_index=True)
-            if not removed:
+            if removed is False:
                 removed_species = random.choice(self.adsorbate_species)
             else:
                 removed_species = ads_sites[removed]['fragment']            
@@ -801,7 +800,7 @@ class ReplaceAdsorbate(AdsorbateOperator):
         Please only use this for nanoparticles.
 
     tilt_angle : float, default 0.
-        Tilt the adsorbate with an angle (in degress) relative to the 
+        Tilt the adsorbate with an angle (in degrees) relative to the 
         surface normal.
 
     subtract_heights : bool, default False
@@ -904,7 +903,7 @@ class ReplaceAdsorbate(AdsorbateOperator):
 
             removed = self.remove_adsorbate(indi, ads_sites,
                                             return_site_index=True)
-            if not removed:
+            if removed is False:
                 removed = random.choice(range(len(ads_sites)))
                 removed_species = 'x'
             else:
@@ -955,7 +954,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
         surface. Choosing adsorbate species with equal probability if 
         not specified.
 
-    vacancy_replacement : bool, default False
+    replace_vacancy : bool, default False
         Whether to allow replacing adsorbates with vacancies, i.e., 
         effectively removing all adsorbates of a certain species.
         Note that if you want to specify species_probabilties, you
@@ -981,7 +980,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
         consistent with the site to which the adsorbate was added. 
 
     tilt_angle : float, default 0.
-        Tilt the adsorbate with an angle (in degress) relative to the 
+        Tilt the adsorbate with an angle (in degrees) relative to the 
         surface normal.
 
     dmax : float, default 2.5
@@ -991,7 +990,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
     """
 
     def __init__(self, adsorbate_species,
-                 vacancy_replacement=False,
+                 replace_vacancy=False,
                  species_probabilities=None,
                  heights=site_heights,
                  adsorption_sites=None,
@@ -1003,7 +1002,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
         self.descriptor = 'ReplaceAdsorbateSpecies'
 
         assert len(set(self.adsorbate_species)) > 1
-        self.vacancy_replacement = vacancy_replacement
+        self.replace_vacancy = replace_vacancy
         self.heights = site_heights 
         for k, v in heights.items():
             self.heights[k] = v
@@ -1059,7 +1058,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
                                                           self.descriptor)
         spec = random.choice(specs)
         all_specs = self.adsorbate_species.copy()
-        if self.vacancy_replacement:
+        if self.replace_vacancy:
             if 'vacancy' not in all_specs:
                 all_specs.append('vacancy')
 
@@ -1096,6 +1095,7 @@ class ReplaceAdsorbateSpecies(AdsorbateOperator):
             for st in ads_sites:
                 if st['indices'] in rmsis:
                     height = self.heights[st['site']]
+#                    height = st['bond_length']
                     add_adsorbate_to_site(indi, to_spec, st, height, tilt_angle=self.tilt_angle) 
         
         if True in indi.pbc:
@@ -1723,128 +1723,47 @@ class SimpleCutSpliceCrossoverWithAdsorbates(AdsorbateOperator):
         return ac
 
 
-class AdsorbateCatalystCrossover(AdsorbateOperator):
-    """Crossover that divides two particles or two slabs by the adsorbate
-    -catalyst interfaces and exchange all adsorbates (only returns one of
-    them). The indexing of the atoms is preserved. Please only use this 
-    operator with other operators that also preserves the indexing.
+class CatalystAdsorbateCrossover(AdsorbateOperator):                                   
+    """Crossover that divides two particles or two slabs by the catalyst-
+    adsorbates interfaces and exchange all adsorbates (only returns one 
+    of them). The indexing of the atoms is preserved. Please only use 
+    this operator with other operators that also preserves the indexing.
 
     The composition or the coverage is fixed if it is preserved by all 
     other operators being used.
 
     Parameters
     ----------
-    adsorbate_species : str or list of strs 
-        One or a list of adsorbate species to be added to the surface.
-
-    heights : dict, default acat.settings.site_heights
-        A dictionary that contains the adsorbate height for each site 
-        type. Use the default height settings if the height for a site 
-        type is not specified.
-
-    adsorption_sites : acat.adsorption_sites.ClusterAdsorptionSites object \
-        or acat.adsorption_sites.SlabAdsorptionSites object, default None
-        Provide the built-in adsorption sites class to accelerate the 
-        genetic algorithm. Make sure all the operators used with this
-        operator preserve the indexing of the atoms.
-
     group_by_sites : bool, default False
         If atoms.info['data']['groups'] is used, please set this to True
         if the groups is for adsorption sites, so that the offspring will
         inheritate the site groups. The default is grouping by slab atoms.
 
-    subtract_heights : bool, default False
-        Whether to subtract the height from the bond length when allocating
-        a site to an adsorbate. Default is to allocate the site that is
-        closest to the adsorbate's binding atom without subtracting height.
-        Useful for ensuring the allocated site for each adsorbate is
-        consistent with the site to which the adsorbate was added. 
-
-    dmax : float, default 2.5
-        The maximum bond length (in Angstrom) between an atom and its
-        nearest site to be considered as the atom being bound to the site.
-
     """
 
-    def __init__(self, adsorbate_species,
-                 heights=site_heights,
-                 adsorption_sites=None,
-                 group_by_sites=False,
-                 subtract_heights=False,
-                 dmax=2.5, **kwargs):
-        AdsorbateOperator.__init__(self, adsorbate_species)
-        self.descriptor = 'AdsorbateCatalystCrossover'
+    def __init__(self, group_by_sites=False):
+        AdsorbateOperator.__init__(self, adsorbate_species='X')
+        self.descriptor = 'CatalystAdsorbateCrossover'
 
-        self.heights = site_heights 
-        for k, v in heights.items():
-            self.heights[k] = v
-        self.adsorption_sites = adsorption_sites
         self.group_by_sites = group_by_sites
-        self.kwargs = {'allow_6fold': False, 'composition_effect': False, 
-                       'ignore_bridge_sites': False, 'label_sites': False}
-        self.kwargs.update(kwargs)
-        if self.adsorption_sites is not None:
-            for k in self.kwargs.keys():             
-                self.kwargs[k] = attrgetter(k)(self.adsorption_sites)
-        self.__dict__.update(self.kwargs)
-        if subtract_heights:
-            self.subtract_heights = self.heights
-        else:
-            self.subtract_heights = None        
         self.min_inputs = 2
-        self.dmax = dmax
         
     def get_new_individual(self, parents):
         f, m = parents        
         indi = f.copy()
-        
-        # Place adsorbates from m onto the clean slab of f
-        if True in indi.pbc:
-            if self.adsorption_sites is not None:
-                sas = self.adsorption_sites
-                sas.update(indi)
-            else:
-                sas = SlabAdsorptionSites(indi, **self.kwargs) 
-            msac = SlabAdsorbateCoverage(m, sas, subtract_heights=
-                                         self.subtract_heights, dmax=self.dmax)
-        else:
-            if self.adsorption_sites is not None:
-                sas = self.adsorption_sites
-                sas.update(indi)                
-            else:
-                sas = ClusterAdsorptionSites(indi, **self.kwargs)
-            msac = ClusterAdsorbateCoverage(m, sas, subtract_heights=
-                                            self.subtract_heights, dmax=self.dmax)
-        indi = indi[sas.indices]
-        mhsl = msac.hetero_site_list
-
-        adsi_dict = {}
-        for st in mhsl:
-            if st['occupied']:
-                si = st['indices']
-                adsi_dict[si] = {}
-                adsi_dict[si]['height'] = self.heights[st['site']]
-                adsi_dict[si]['fragment'] = st['fragment']
-                adsi_dict[si]['fragment_indices'] = st['fragment_indices']        
-
-        for st in sas.site_list:
-            si = st['indices']
-            if si in adsi_dict:
-                adsorbate = adsi_dict[si]['fragment']
-                height = adsi_dict[si]['height']    
-                add_adsorbate_to_site(indi, adsorbate, st, height)
+        slab_ids = [a.index for a in indi if a.symbol not in adsorbate_elements]
+        indi.symbols[slab_ids] = m.symbols[slab_ids]
 
         indi = self.initialize_individual(f, indi)
         if 'groups' in f.info['data']:
             if self.group_by_sites:
-                indi.info['data']['groups'] = m.info['data']['groups']
-            else:
                 indi.info['data']['groups'] = f.info['data']['groups']
+            else:
+                indi.info['data']['groups'] = m.info['data']['groups']
         indi.info['data']['parents'] = [i.info['confid'] for i in parents] 
         indi.info['data']['operation'] = 'crossover'
-        indi.info['data']['adsorbates'] = [t[0] for t in 
-            msac.get_adsorbates(self.adsorbate_species)]
         parent_message = ':Parents {0} {1}'.format(f.info['confid'],
                                                    m.info['confid'])
         return (self.finalize_individual(indi),
                 self.descriptor + parent_message)
+
